@@ -11,6 +11,9 @@ import logging
 import threading
 import traceback
 
+logging.warning("The 0.2.x branch become deprecated.")
+logging.warning("Details: http://docs.pyroute2.org/migration.html")
+
 try:
     basestring = basestring
 except NameError:
@@ -229,6 +232,21 @@ def uuid32():
                             os.getpid()))
 
 
+def list_subnet(dqn, mask, family=socket.AF_INET):
+    '''
+    List all IPs in the network
+    '''
+    if family != socket.AF_INET:
+        raise NotImplementedError('please report the issue')
+
+    ret = []
+    net = struct.unpack('>I', socket.inet_pton(family, dqn))[0]
+    shift = 32 - mask
+    for host in range(1, 2 ** shift):
+        ret.append(socket.inet_ntop(family, struct.pack('>I', net | host)))
+    return ret
+
+
 def dqn2int(mask):
     '''
     IPv4 dotted quad notation to int mask conversion
@@ -246,3 +264,40 @@ def hexdump(payload, length=0):
     else:
         return ':'.join('{0:02x}'.format(ord(c))
                         for c in payload[:length] or payload)
+
+
+class PipeSocket(object):
+    '''
+    Socket-like object for one-system IPC.
+
+    It is netlink-specific, since relies on length value
+    provided in the first four bytes of each message.
+    '''
+
+    family = AF_PIPE
+
+    def __init__(self, rfd, wfd):
+        self.rfd = rfd
+        self.wfd = wfd
+
+    def send(self, data):
+        return os.write(self.wfd, data)
+
+    def sendto(self, data, addr):
+        return self.send(data)
+
+    def recv(self, length=0, flags=0):
+        ret = os.read(self.rfd, 4)
+        length = struct.unpack('I', ret)[0]
+        ret += os.read(self.rfd, length - 4)
+        return ret
+
+    def getsockname(self):
+        return self.rfd, self.wfd
+
+    def fileno(self):
+        return self.rfd
+
+    def close(self):
+        os.close(self.rfd)
+        os.close(self.wfd)
