@@ -7,13 +7,14 @@ import os
 import io
 import uuid
 import ssl
+import sys
 
-from pyroute2.common import AF_PIPE
 from pyroute2.netlink import Marshal
 from pyroute2.netlink import NLMSG_CONTROL
 from pyroute2.netlink import NLMSG_TRANSPORT
 from pyroute2.netlink import IPRCMD_ERR
 from pyroute2.netlink import IPRCMD_ACK
+from pyroute2.common import PipeSocket
 from pyroute2.netlink.generic import mgmtmsg
 from pyroute2.netlink.generic import envmsg
 from pyroute2.iocore import NLT_CONTROL
@@ -58,40 +59,6 @@ def _monkey_handshake(self):
 ssl.SSLSocket.do_handshake = _monkey_handshake
 
 
-class PipeSocket(object):
-    '''
-    Socket-like object for one-system IPC.
-
-    It is netlink-specific, since relies on length value
-    provided in the first four bytes of each message.
-    '''
-
-    family = AF_PIPE
-
-    def __init__(self, rfd, wfd):
-        self.rfd = rfd
-        self.wfd = wfd
-
-    def send(self, data):
-        os.write(self.wfd, data)
-
-    def recv(self, length=0, flags=0):
-        ret = os.read(self.rfd, 4)
-        length = struct.unpack('I', ret)[0]
-        ret += os.read(self.rfd, length - 4)
-        return ret
-
-    def getsockname(self):
-        return self.rfd, self.wfd
-
-    def fileno(self):
-        return self.rfd
-
-    def close(self):
-        os.close(self.rfd)
-        os.close(self.wfd)
-
-
 def pairPipeSockets():
     pipe0_r, pipe0_w = os.pipe()
     pipe1_r, pipe1_w = os.pipe()
@@ -234,7 +201,8 @@ class IOBroker(object):
         self._expire_thread = Cache(self._stop_event)
         self._expire_thread.register_map(self.masquerade, self.nonces.free)
         self._expire_thread.register_map(self.packet_ids)
-        self._expire_thread.setDaemon(True)
+        if hasattr(sys, 'ps1'):
+            self._expire_thread.setDaemon(True)
         if ioloop:
             self.ioloop = ioloop
             self.standalone = False
