@@ -56,12 +56,13 @@ class IOCore(object):
 
     def __init__(self, debug=False, timeout=3, do_connect=False,
                  host=None, key=None, cert=None, ca=None,
-                 addr=None, fork=False, secret=None):
+                 addr=None, fork=False, secret=None, pid=None):
         addr = addr or uuid32()
         self._timeout = timeout
         self.default_broker = addr
         self.default_dport = 0
         self.uids = set()
+        self.pid = pid
         self.listeners = {}     # {nonce: Queue(), ...}
         self.callbacks = []     # [(predicate, callback, args), ...]
         self.debug = debug
@@ -294,12 +295,15 @@ class IOCore(object):
 
     def connect(self, host=None, key='', cert='', ca='', addr=None):
         host = host or self.host
+        attrs = [['IPR_ATTR_HOST', host],
+                 ['IPR_ATTR_SSL_KEY', key],
+                 ['IPR_ATTR_SSL_CERT', cert],
+                 ['IPR_ATTR_SSL_CA', ca]]
+        if self.pid is not None:
+            attrs.append(['IPR_ATTR_PID', self.pid])
         (uid,
          peer) = self.command(IPRCMD_CONNECT,
-                              [['IPR_ATTR_HOST', host],
-                               ['IPR_ATTR_SSL_KEY', key],
-                               ['IPR_ATTR_SSL_CERT', cert],
-                               ['IPR_ATTR_SSL_CA', ca]],
+                              attrs,
                               expect=['IPR_ATTR_UUID',
                                       'IPR_ATTR_ADDR'],
                               addr=addr)
@@ -481,11 +485,11 @@ class IOCore(object):
         return result
 
     @debug
-    def push(self, host, msg,
+    def push(self, msg, host=None,
              env_flags=None,
              nonce=0,
              cname=None):
-        addr, port = host
+        addr, port = host or (self.default_broker, self.default_dport)
         envelope = envmsg()
         envelope['header']['sequence_number'] = nonce
         envelope['header']['pid'] = os.getpid()
@@ -517,7 +521,7 @@ class IOCore(object):
         port = port or self.default_dport
         addr = addr or self.default_broker
         self.listeners[nonce] = Queue.Queue(maxsize=_QUEUE_MAXSIZE)
-        self.push((addr, port), msg, env_flags, nonce, cname)
+        self.push(msg, (addr, port), env_flags, nonce, cname)
         return self.get(nonce,
                         nonce_pool=nonce_pool,
                         timeout=response_timeout,
