@@ -5,6 +5,8 @@ from pyroute2.netlink import NetlinkSocket
 from pyroute2.netlink import IPRCMD_CONNECT
 from pyroute2.netlink.generic import envmsg
 from pyroute2.netlink.generic import mgmtmsg
+from pyroute2.netlink.generic import NETLINK_ROUTE
+from pyroute2.netlink.rtnl import RtnlSocket
 from pyroute2.iocore.utils import get_socket
 from pyroute2.iocore.utils import access
 try:
@@ -22,6 +24,7 @@ def command(broker, sock, env, cmd, rsp):
     key = cmd.get_attr('IPR_ATTR_SSL_KEY')
     cert = cmd.get_attr('IPR_ATTR_SSL_CERT')
     ca = cmd.get_attr('IPR_ATTR_SSL_CA')
+    pid = cmd.get_attr('IPR_ATTR_PID')
 
     target = urlparse.urlparse(url)
     peer = broker.addr
@@ -39,7 +42,11 @@ def command(broker, sock, env, cmd, rsp):
 
     elif target.scheme == 'netlink':
         res = target.path.split("/")
-        new_sock = NetlinkSocket(int(res[1]))
+        family = int(res[1])
+        if family == NETLINK_ROUTE:
+            new_sock = RtnlSocket()
+        else:
+            new_sock = NetlinkSocket(int(res[1]), pid=pid)
         new_sock.bind(int(res[2]))
         gate = lambda d, s:\
             new_sock.sendto(broker.gate_untag(d, s), (0, 0))
@@ -90,8 +97,10 @@ def command(broker, sock, env, cmd, rsp):
     rsp['attrs'].append(['IPR_ATTR_UUID', uid])
     rsp['attrs'].append(['IPR_ATTR_ADDR', peer])
     try:
-        broker.ioloop.register(new_sock, route,
-                               defer=True)
+        broker.ioloop.register(new_sock, route, defer=True)
+        if hasattr(new_sock, 'bypass'):
+            broker.ioloop.register(new_sock.bypass, route, defer=True,
+                                   proxy=new_sock)
     except Exception as e:
         if e.errno != errno.EEXIST:
             raise e
