@@ -13,7 +13,59 @@ from pyroute2.common import basestring
 _letters = re.compile('[A-Za-z]')
 _fmt_letters = re.compile('[^!><@=][!><@=]')
 
-#  Netlink family
+
+NLMSG_MIN_TYPE = 0x10
+
+GENL_NAMSIZ = 16    # length of family name
+GENL_MIN_ID = NLMSG_MIN_TYPE
+GENL_MAX_ID = 1023
+
+GENL_ADMIN_PERM = 0x01
+GENL_CMD_CAP_DO = 0x02
+GENL_CMD_CAP_DUMP = 0x04
+GENL_CMD_CAP_HASPOL = 0x08
+
+#
+# List of reserved static generic netlink identifiers:
+#
+GENL_ID_GENERATE = 0
+GENL_ID_CTRL = NLMSG_MIN_TYPE
+
+#
+# Controller
+#
+
+CTRL_CMD_UNSPEC = 0x0
+CTRL_CMD_NEWFAMILY = 0x1
+CTRL_CMD_DELFAMILY = 0x2
+CTRL_CMD_GETFAMILY = 0x3
+CTRL_CMD_NEWOPS = 0x4
+CTRL_CMD_DELOPS = 0x5
+CTRL_CMD_GETOPS = 0x6
+CTRL_CMD_NEWMCAST_GRP = 0x7
+CTRL_CMD_DELMCAST_GRP = 0x8
+CTRL_CMD_GETMCAST_GRP = 0x9  # unused
+
+
+CTRL_ATTR_UNSPEC = 0x0
+CTRL_ATTR_FAMILY_ID = 0x1
+CTRL_ATTR_FAMILY_NAME = 0x2
+CTRL_ATTR_VERSION = 0x3
+CTRL_ATTR_HDRSIZE = 0x4
+CTRL_ATTR_MAXATTR = 0x5
+CTRL_ATTR_OPS = 0x6
+CTRL_ATTR_MCAST_GROUPS = 0x7
+
+CTRL_ATTR_OP_UNSPEC = 0x0
+CTRL_ATTR_OP_ID = 0x1
+CTRL_ATTR_OP_FLAGS = 0x2
+
+CTRL_ATTR_MCAST_GRP_UNSPEC = 0x0
+CTRL_ATTR_MCAST_GRP_NAME = 0x1
+CTRL_ATTR_MCAST_GRP_ID = 0x2
+
+
+#  Different Netlink families
 #
 NETLINK_ROUTE = 0            # Routing/device hook
 NETLINK_UNUSED = 1           # Unused number
@@ -307,7 +359,14 @@ class nlmsg_base(dict):
                         if field[1] == 'z' and self[name][-1] in (0x00, '\0'):
                             self[name] = self[name][:-1]
                     else:
-                        self[name] = value
+                        if self.pack == 'struct':
+                            names = name.split(',')
+                            values = list(value)
+                            for name in names:
+                                if name[0] != '_':
+                                    self[name] = values.pop(0)
+                        else:
+                            self[name] = value
 
                 else:
                     # FIXME: log an error
@@ -359,7 +418,9 @@ class nlmsg_base(dict):
                         value = int(value)
 
                 try:
-                    if type(value) in (list, tuple, set):
+                    if fmt[-1] == 'x':
+                        payload += struct.pack(fmt)
+                    elif type(value) in (list, tuple, set):
                         payload += struct.pack(fmt, *value)
                     else:
                         payload += struct.pack(fmt, value)
@@ -707,6 +768,13 @@ class ctrlmsg(genlmsg):
                ('CTRL_ATTR_MCAST_GROUPS', 'hex'))
 
 
+class cdatamsg(genlmsg):
+    '''
+    Simplest netlink CDATA container
+    '''
+    nla_map = (('IPR_ATTR_CDATA', 'cdata'), )
+
+
 class mgmtmsg(ctrlmsg):
     '''
     Messaging system control messages
@@ -721,7 +789,8 @@ class mgmtmsg(ctrlmsg):
                ('IPR_ATTR_CID', 'uint32'),
                ('IPR_ATTR_KEY', 'u32key'),
                ('IPR_ATTR_UUID', 'asciiz'),
-               ('IPR_ATTR_SOURCE', 'uint32'))
+               ('IPR_ATTR_SOURCE', 'uint32'),
+               ('IPR_ATTR_PID', 'uint32'))
 
     class u32key(nla):
         fields = (('offset', 'I'),
