@@ -193,7 +193,9 @@ and `vlan` interfaces. VLAN creation requires also `link` and
 
 '''
 import os
+import sys
 import uuid
+import time
 import socket
 import threading
 
@@ -202,13 +204,17 @@ from socket import AF_INET
 from socket import AF_INET6
 from pyroute2.common import dqn2int
 from pyroute2.common import Dotkeys
+from pyroute2.common import basestring
 from pyroute2.iocore import TimeoutError
 from pyroute2.netlink import NetlinkError
 from pyroute2.netlink.ipdb import compat
+from pyroute2.netlink.iproute import IFF_MASK
 from pyroute2.netlink.iproute import IPRoute
 from pyroute2.netlink.rtnl.rtmsg import rtmsg
 from pyroute2.netlink.rtnl.ifinfmsg import ifinfmsg
 from pyroute2.netlink.rtnl.tcmsg import tcmsg
+from pyroute2.netlink.rtnl.brmsg import brmsg
+from pyroute2.netlink.rtnl.bomsg import bomsg
 
 tc_fields = [tcmsg.nla2name(i[0]) for i in tcmsg.nla_map]
 
@@ -222,6 +228,10 @@ _FAIL_ROLLBACK = 0b00000010
 _FAIL_MASK = 0b11111111
 
 
+class DeprecationException(Exception):
+    pass
+
+
 class CommitException(Exception):
     pass
 
@@ -339,6 +349,10 @@ class LinkedSet(set):
             else:
                 self._ct = set(value)
                 self.target.clear()
+                # immediately check, if the target already
+                # reached -- otherwise you will miss the
+                # target forever
+                self.check_target()
 
     def check_target(self):
         '''
@@ -514,6 +528,10 @@ class IPRouteRequest(IPRequest):
     '''
 
     def __setitem__(self, key, value):
+        # fix family
+        if isinstance(value, basestring) and value.find(':') >= 0:
+            self['family'] = AF_INET6
+        # work on the rest
         if (key == 'dst') and (value != 'default'):
             value = value.split('/')
             if len(value) == 1:
@@ -530,6 +548,36 @@ class IPRouteRequest(IPRequest):
             dict.__setitem__(self, key, value)
 
 
+class CBRequest(IPRequest):
+    '''
+    FIXME
+    '''
+    commands = None
+    msg = None
+
+    def __init__(self, *argv, **kwarg):
+        self['commands'] = {'attrs': []}
+
+    def __setitem__(self, key, value):
+        if value is None:
+            return
+        if key in self.commands:
+            self['commands']['attrs'].\
+                append([self.msg.name2nla(key), value])
+        else:
+            dict.__setitem__(self, key, value)
+
+
+class BridgeRequest(CBRequest):
+    commands = [brmsg.nla2name(i[0]) for i in brmsg.commands.nla_map]
+    msg = brmsg
+
+
+class BondRequest(CBRequest):
+    commands = [bomsg.nla2name(i[0]) for i in bomsg.commands.nla_map]
+    msg = bomsg
+
+
 class IPLinkRequest(IPRequest):
     '''
     Utility class, that converts human-readable dictionary
@@ -537,6 +585,18 @@ class IPLinkRequest(IPRequest):
     '''
 
     def __setitem__(self, key, value):
+        # there must be no "None" values in the request
+        if value is None:
+            return
+
+        # all the values must be in ascii
+        try:
+            if isinstance(value, unicode):
+                value = value.encode('ascii')
+        except NameError:
+            pass
+
+        # set up specific keys
         if key == 'kind':
             if 'IFLA_LINKINFO' not in self:
                 self['IFLA_LINKINFO'] = {'attrs': []}
@@ -569,10 +629,12 @@ class Transactional(Dotkeys):
         self.uid = uuid.uuid4()
         self.ipdb = ipdb
         self.last_error = None
-        self._callbacks = []
+        self._commit_hooks = []
         self._fields = []
         self._tids = []
         self._transactions = {}
+        self._sids = []
+        self._snapshots = {}
         self._targets = {}
         self._local_targets = {}
         self._mode = mode or ipdb.mode
@@ -581,12 +643,23 @@ class Transactional(Dotkeys):
         self._linked_sets = set()
 
     def register_callback(self, callback):
-        self._callbacks.append(callback)
+        raise DeprecationException("deprecated since 0.2.15;"
+                                   "use `register_commit_hook()`")
+
+    def register_commit_hook(self, hook):
+        # FIXME: write docs
+        self._commit_hooks.append(hook)
 
     def unregister_callback(self, callback):
-        for cb in tuple(self._callbacks):
-            if callback == cb:
-                self._callbacks.pop(self._callbacks.index(cb))
+        raise DeprecationException("deprecated since 0.2.15;"
+                                   "use `unregister_commit_hook()`")
+
+    def unregister_commit_hook(self, hook):
+        # FIXME: write docs
+        with self._write_lock:
+            for cb in tuple(self._commit_hooks):
+                if hook == cb:
+                    self._commit_hooks.pop(self._commit_hooks.index(cb))
 
     def pick(self, detached=True):
         '''
@@ -648,23 +721,68 @@ class Transactional(Dotkeys):
                 res[key] = diff
         return res
 
+    def dump(self, not_none=True):
+        with self._write_lock:
+            res = {}
+            for key in self:
+                if self[key] is not None and key[0] != '_':
+                    if isinstance(self[key], Transactional):
+                        res[key] = self[key].dump()
+                    elif isinstance(self[key], LinkedSet):
+                        res[key] = tuple(self[key])
+                    else:
+                        res[key] = self[key]
+            return res
+
+    def load(self, data):
+        pass
+
     def commit(self, *args, **kwarg):
         pass
 
+    def last_snapshot_id(self):
+        return self._sids[-1]
+
+    def revert(self, sid):
+        with self.ipdb.exclusive:
+            self._transactions[sid] = self._snapshots[sid]
+            self._tids.append(sid)
+            self._sids.remove(sid)
+            del self._snapshots[sid]
+            return self
+
+    def snapshot(self):
+        '''
+        Create new snapshot
+        '''
+        return self._begin(mapping=self._snapshots,
+                           ids=self._sids,
+                           detached=True)
+
     def begin(self):
         '''
         Start new transaction
         '''
+        return self._begin(mapping=self._transactions,
+                           ids=self._tids,
+                           detached=False)
+
+    def _begin(self, mapping, ids, detached):
         # keep snapshot's ip addr set updated from the OS
         # it is required by the commit logic
         with self.ipdb.exclusive:
             if self.ipdb._stop:
                 raise RuntimeError("Can't start transaction on released IPDB")
-            t = self.pick(detached=False)
-            self._transactions[t.uid] = t
-            self._tids.append(t.uid)
+            t = self.pick(detached=detached)
+            mapping[t.uid] = t
+            ids.append(t.uid)
             return t.uid
 
+    def last_snapshot(self):
+        if not self._sids:
+            raise TypeError('create a snapshot first')
+        return self._snapshots[self._sids[-1]]
+
     def last(self):
         '''
         Return last open transaction
@@ -689,11 +807,17 @@ class Transactional(Dotkeys):
             del added[key]
         return added
 
-    def drop(self):
+    def drop(self, tid=None):
         '''
-        Drop all not applied changes and rollback transaction.
+        Drop a transaction.
         '''
-        del self._transactions[self._tids.pop()]
+        with self.ipdb.exclusive:
+            if isinstance(tid, Transactional):
+                tid = tid.uid
+            elif tid is None:
+                tid = self._tids[-1]
+            self._tids.pop(self._tids.index(tid))
+            del self._transactions[tid]
 
     @update
     def __setitem__(self, direct, key, value):
@@ -704,6 +828,9 @@ class Transactional(Dotkeys):
             transaction[key] = value
             transaction._targets[key] = threading.Event()
         else:
+            # set the item
+            Dotkeys.__setitem__(self, key, value)
+
             # update on local targets
             if key in self._local_targets:
                 func = self._fields_cmp.get(key, lambda x, y: x == y)
@@ -711,15 +838,12 @@ class Transactional(Dotkeys):
                     self._local_targets[key].set()
 
             # cascade update on nested targets
-            for tn in self._transactions.values():
-                if key in tn._targets:
+            for tn in tuple(self._transactions.values()):
+                if (key in tn._targets) and (key in tn):
                     if self._fields_cmp.\
                             get(key, lambda x, y: x == y)(value, tn[key]):
                         tn._targets[key].set()
 
-            # and, finally, set the item :)
-            Dotkeys.__setitem__(self, key, value)
-
     @update
     def __delitem__(self, direct, key):
         # firstly set targets
@@ -762,7 +886,7 @@ class TrafficControl(Transactional):
         Transactional.__init__(self, ipdb, mode)
         self._fields = tc_fields
 
-    def load(self, msg):
+    def load_netlink(self, msg):
         with self._direct_state:
             self.update(msg)
             self['kind'] = msg.get_attr('TCA_KIND')
@@ -798,7 +922,7 @@ class Interface(Transactional):
     exception will be raised. Failed transaction review
     will be attached to the exception.
     '''
-    _fields_cmp = {'flags': lambda x, y: x & y}
+    _fields_cmp = {'flags': lambda x, y: x & y & IFF_MASK == y & IFF_MASK}
 
     def __init__(self, ipdb, mode=None):
         '''
@@ -820,14 +944,28 @@ class Interface(Transactional):
         self._exists = False
         self._flicker = False
         self._virtual_fields = ('removal', 'flicker', 'state')
-        self._fields = [ifinfmsg.nla2name(i[0]) for i in ifinfmsg.nla_map]
-        self._fields.append('flags')
-        self._fields.append('mask')
-        self._fields.append('change')
+        self._xfields = {'common': [ifinfmsg.nla2name(i[0]) for i
+                                    in ifinfmsg.nla_map],
+                         'bridge': [brmsg.nla2name(i[0]) for i
+                                    in brmsg.commands.nla_map],
+                         'bond': [bomsg.nla2name(i[0]) for i
+                                  in bomsg.commands.nla_map]}
+        self._xfields['bridge'].append('index')
+        self._xfields['bond'].append('index')
+        self._xfields['common'].append('index')
+        self._xfields['common'].append('flags')
+        self._xfields['common'].append('mask')
+        self._xfields['common'].append('change')
+        self._xfields['common'].append('kind')
+        self._xfields['common'].append('vlan_id')
+        self._xfields['common'].append('bond_mode')
+        for ftype in self._xfields:
+            self._fields += self._xfields[ftype]
         self._fields.extend(self._virtual_fields)
         self._load_event = threading.Event()
         self._linked_sets.add('ipaddr')
         self._linked_sets.add('ports')
+        self._freeze = None
         # 8<-----------------------------------
         # local setup: direct state is required
         with self._direct_state:
@@ -847,11 +985,79 @@ class Interface(Transactional):
         '''
         [property] Link to the parent interface -- if it exists
         '''
-        ret = [self[i] for i in ('link', 'master')
-               if (i in self) and isinstance(self[i], int)] or [None]
-        return ret[0]
+        return self.get('master', None)
 
-    def load(self, dev):
+    def freeze(self):
+        dump = self.dump()
+
+        def cb(ipdb, msg, action):
+            if msg.get('index', -1) == dump['index']:
+                with ipdb.exclusive:
+                    tr = self.load(dump)
+                    for _ in range(3):
+                        try:
+                            self.commit(transaction=tr)
+                        except (CommitException, RuntimeError):
+                            # ignore here both CommitExceptions
+                            # and RuntimeErrors (aka rollback errors),
+                            # since ususally it is a races between
+                            # 3d party setup and freeze; just
+                            # sliently try again for several times
+                            continue
+                        break
+
+        self._freeze = cb
+        self.ipdb.register_callback(self._freeze)
+        return self
+
+    def unfreeze(self):
+        self.ipdb.unregister_callback(self._freeze)
+        self._freeze = None
+        return self
+
+    def load(self, data):
+        with self._write_lock:
+            template = self.__class__(ipdb=self.ipdb, mode='snapshot')
+            template.load_dict(data)
+            return template
+
+    def load_dict(self, data):
+        with self._direct_state:
+            for key in data:
+                if key == 'ipaddr':
+                    for addr in data[key]:
+                        if isinstance(addr, basestring):
+                            addr = (addr, )
+                        self.add_ip(*addr)
+                elif key == 'ports':
+                    for port in data[key]:
+                        self.add_port(port)
+                else:
+                    self[key] = data[key]
+
+    def load_bridge(self):
+        '''
+        '''
+        with self._direct_state:
+            dev = self.nl.get_bridge(self['index'], self['ifname'])[0]
+            attrs = dev.get_attr('IFBO_COMMANDS',
+                                 {'attrs': []}).get('attrs', [])
+            for (name, value) in attrs:
+                norm = brmsg.nla2name(name)
+                self[norm] = value
+
+    def load_bond(self):
+        '''
+        '''
+        with self._direct_state:
+            dev = self.nl.get_bond(self['index'], self['ifname'])[0]
+            attrs = dev.get_attr('IFBO_COMMANDS',
+                                 {'attrs': []}).get('attrs', [])
+            for (name, value) in attrs:
+                norm = bomsg.nla2name(name)
+                self[norm] = value
+
+    def load_netlink(self, dev):
         '''
         Update the interface info from RTM_NEWLINK message.
 
@@ -885,6 +1091,11 @@ class Interface(Transactional):
                 kind = get_interface_type(self['ifname'])
                 if kind is not False:
                     self['kind'] = kind
+            # poll bridge & bond info
+            if self.get('kind', None) == 'bridge':
+                self.load_bridge()
+            elif self.get('kind', None) == 'bond':
+                self.load_bond()
             # finally, cleanup all not needed
             for item in self.cleanup:
                 if item in self:
@@ -951,7 +1162,6 @@ class Interface(Transactional):
             transaction.add_port(port)
         else:
             self['ports'].unlink(port)
-            compat.fix_add_master(self.ipdb.interfaces[port], self)
             self['ports'].add(port)
         return self
 
@@ -968,8 +1178,6 @@ class Interface(Transactional):
                 transaction.del_port(port)
         else:
             self['ports'].unlink(port)
-            compat.fix_del_master(self.ipdb.interfaces[port])
-            # FIXME: do something with it, please
             self['ports'].remove(port)
         return self
 
@@ -990,6 +1198,13 @@ class Interface(Transactional):
         self._load_event.wait(_SYNC_TIMEOUT)
         return self
 
+    def filter(self, ftype):
+        ret = {}
+        for key in self:
+            if key in self._xfields[ftype]:
+                ret[key] = self[key]
+        return ret
+
     def commit(self, tid=None, transaction=None, rollback=False):
         '''
         Commit transaction. In the case of exception all
@@ -999,14 +1214,18 @@ class Interface(Transactional):
             error = None
             added = None
             removed = None
+            drop = True
             if tid:
                 transaction = self._transactions[tid]
             else:
-                transaction = transaction or self.last()
+                if transaction:
+                    drop = False
+                else:
+                    transaction = self.last()
 
             # if the interface does not exist, create it first ;)
             if not self._exists:
-                request = IPLinkRequest(self)
+                request = IPLinkRequest(self.filter('common'))
                 self.ipdb._links_event.clear()
 
                 # create watchdog
@@ -1020,7 +1239,7 @@ class Interface(Transactional):
                         del request['broadcast']
                     # 8<----------------------------------------------------
                     try:
-                        compat.fix_create_link(self.nl, request)
+                        self.nl.link('add', **request)
                     except NetlinkError as x:
                         # Operation not supported
                         if x.code == 95 and request.get('index', 0) != 0:
@@ -1028,7 +1247,7 @@ class Interface(Transactional):
                             request = IPLinkRequest({'ifname': self['ifname'],
                                                      'kind': self['kind'],
                                                      'index': 0})
-                            compat.fix_create_link(self.nl, request)
+                            self.nl.link('add', **request)
                         else:
                             raise
                 except Exception:
@@ -1081,7 +1300,46 @@ class Interface(Transactional):
                 for i in added['ipaddr']:
                     self.nl.addr('add', self['index'], i[0], i[1])
 
+                    # 8<--------------------------------------
+                    # FIXME: kernel bug, sometimes `addr add` for
+                    # bond interfaces returns success, but does
+                    # really nothing
+
+                    if self['kind'] == 'bond':
+                        while True:
+                            try:
+                                # dirtiest hack, but we have to use it here
+                                time.sleep(0.1)
+                                self.nl.addr('add', self['index'], i[0], i[1])
+                            # continue to try to add the address
+                            # until the kernel reports `file exists`
+                            #
+                            # a stupid solution, but must help
+                            except NetlinkError as e:
+                                if e.code == 17:
+                                    break
+                                else:
+                                    raise
+                            except Exception:
+                                raise
+                    # 8<--------------------------------------
+
                 if removed['ipaddr'] or added['ipaddr']:
+                    # 8<--------------------------------------
+                    # bond and bridge interfaces do not send
+                    # IPv6 address updates, when are down
+                    #
+                    # beside of that, bridge interfaces are
+                    # down by default, so they never send
+                    # address updates from beginning
+                    #
+                    # so if we need, force address load
+                    #
+                    # FIXME: probably, we should handle other
+                    # types as well
+                    if self['kind'] in ('bond', 'bridge'):
+                        self.nl.get_addr()
+                    # 8<--------------------------------------
                     self['ipaddr'].target.wait(_SYNC_TIMEOUT)
                     if not self['ipaddr'].target.is_set():
                         raise CommitException('ipaddr target is not set')
@@ -1095,20 +1353,27 @@ class Interface(Transactional):
                     port = self.ipdb.interfaces[i]
                     port.set_target('master', None)
                     port.mirror_target('master', 'link')
-                    compat.fix_del_port(self.nl, self, port)
+                    self.nl.link('set', index=port['index'], master=0)
 
                 for i in added['ports']:
                     # enslave the port
                     port = self.ipdb.interfaces[i]
                     port.set_target('master', self['index'])
                     port.mirror_target('master', 'link')
-                    compat.fix_add_port(self.nl, self, port)
+                    self.nl.link('set',
+                                 index=port['index'],
+                                 master=self['index'])
 
                 if removed['ports'] or added['ports']:
                     self.nl.get_links(*(removed['ports'] | added['ports']))
                     self['ports'].target.wait(_SYNC_TIMEOUT)
                     if not self['ports'].target.is_set():
                         raise CommitException('ports target is not set')
+
+                    # RHEL 6.5 compat fix -- an explicit timeout
+                    # it gives a time for all the messages to pass
+                    compat.fix_timeout(1)
+
                     # wait for proper targets on ports
                     for i in list(added['ports']) + list(removed['ports']):
                         port = self.ipdb.interfaces[i]
@@ -1127,12 +1392,38 @@ class Interface(Transactional):
                 # Interface changes
                 request = IPLinkRequest()
                 for key in added:
-                    if key in self._fields:
+                    if key in self._xfields['common']:
                         request[key] = added[key]
+                request['index'] = self['index']
 
                 # apply changes only if there is something to apply
-                if any([request[item] is not None for item in request]):
-                    self.nl.link('set', index=self['index'], **request)
+                if any([request[item] is not None for item in request
+                        if item != 'index']):
+                    self.nl.link('set', **request)
+
+                # bridge changes
+                if self['kind'] == 'bridge':
+                    brq = BridgeRequest()
+                    for key in added:
+                        if key in self._xfields['bridge']:
+                            brq[key] = added[key]
+
+                    brq['index'] = self['index']
+                    brq['ifname'] = self['ifname']
+                    self.nl.setbr(**brq)
+                    self.load_bridge()
+
+                # bridge changes
+                if self['kind'] == 'bond':
+                    boq = BondRequest()
+                    for key in added:
+                        if key in self._xfields['bond']:
+                            boq[key] = added[key]
+
+                    boq['index'] = self['index']
+                    boq['ifname'] = self['ifname']
+                    self.nl.setbo(**boq)
+                    self.load_bond()
 
                 # reload interface to hit targets
                 if transaction._targets:
@@ -1152,13 +1443,14 @@ class Interface(Transactional):
                                             ifname=self['ifname'])
                     if added.get('flicker'):
                         self._flicker = True
-                    compat.fix_del_link(self.nl, self)
+                    self.nl.link('delete', **self)
                     wd.wait()
                     if added.get('flicker'):
                         self._exists = False
                     if added.get('removal'):
                         self._mode = 'invalid'
-                    self.drop()
+                    if drop:
+                        self.drop()
                     return self
                 # 8<---------------------------------------------
 
@@ -1168,9 +1460,9 @@ class Interface(Transactional):
 
                     # 8<-----------------------------------------
                     # Iterate callback chain
-                    for cb in self._callbacks:
+                    for ch in self._commit_hooks:
                         # An exception will rollback the transaction
-                        cb(snapshot, transaction)
+                        ch(self.dump(), snapshot.dump(), transaction.dump())
                     # 8<-----------------------------------------
 
                     assert _FAIL_COMMIT & _FAIL_MASK
@@ -1194,7 +1486,8 @@ class Interface(Transactional):
                     # somethig went wrong during automatic rollback.
                     # that's the worst case, but it is still possible,
                     # since we have no locks on OS level.
-                    self.drop()
+                    if drop:
+                        self.drop()
                     self['ipaddr'].set_target(None)
                     self['ports'].set_target(None)
                     # reload all the database -- it can take a long time,
@@ -1212,7 +1505,7 @@ class Interface(Transactional):
                     raise x
 
             # if it is not a rollback turn
-            if not rollback:
+            if drop and not rollback:
                 # drop last transaction in any case
                 self.drop()
 
@@ -1280,7 +1573,7 @@ class Route(Transactional):
                         'header',
                         'event')
 
-    def load(self, msg):
+    def load_netlink(self, msg):
         with self._direct_state:
             self._exists = True
             self.update(msg)
@@ -1387,7 +1680,7 @@ class RoutingTables(dict):
         route.begin()
         return route
 
-    def load(self, msg):
+    def load_netlink(self, msg):
         '''
         Loads an existing route from a rtmsg
         '''
@@ -1403,10 +1696,10 @@ class RoutingTables(dict):
 
         if key in self.tables[table]:
             ret = self.tables[table][key]
-            ret.load(msg)
+            ret.load_netlink(msg)
         else:
             ret = Route(ipdb=self.ipdb)
-            ret.load(msg)
+            ret.load_netlink(msg)
             self.tables[table][key] = ret
         return ret
 
@@ -1534,7 +1827,8 @@ class IPDB(object):
 
         # start monitoring thread
         self._mthread = threading.Thread(target=self.serve_forever)
-        self._mthread.setDaemon(True)
+        if hasattr(sys, 'ps1'):
+            self._mthread.setDaemon(True)
         self._mthread.start()
 
     def __enter__(self):
@@ -1611,7 +1905,9 @@ class IPDB(object):
             raise KeyError('Unknown callback mode')
         for cb in tuple(cbchain):
             if callback == cb:
-                cbchain.pop(cbchain.index(cb))
+                for t in tuple(self._cb_threads):
+                    t.join(3)
+                return cbchain.pop(cbchain.index(cb))
 
     def release(self):
         '''
@@ -1733,7 +2029,8 @@ class IPDB(object):
             # for interfaces, created by IPDB
             self.ipaddr[index] = IPaddrSet()
 
-        device.load(msg)
+        device.load_netlink(msg)
+
         if not skip_slaves:
             self.update_slaves(msg)
 
@@ -1746,14 +2043,10 @@ class IPDB(object):
 
     def update_routes(self, routes):
         for msg in routes:
-            self.routes.load(msg)
+            self.routes.load_netlink(msg)
 
     def _lookup_master(self, msg):
-        index = msg['index']
-        master = msg.get_attr('IFLA_MASTER') or \
-            msg.get_attr('IFLA_LINK')
-        if (master is None):
-            master = compat.fix_lookup_master(self.interfaces[index])
+        master = msg.get_attr('IFLA_MASTER')
         return self.interfaces.get(master, None)
 
     def update_slaves(self, msg):
@@ -1792,8 +2085,6 @@ class IPDB(object):
             if master is not None:
                 if 'master' in device:
                     device.del_item('master')
-                if 'link' in device:
-                    device.del_item('link')
                 if (master in self.interfaces) and \
                         (msg['index'] in self.interfaces[master].ports):
                     self.interfaces[master].del_port(msg['index'],
