@@ -15,67 +15,15 @@ usable.
 '''
 
 from pyroute2.iocore import TimeoutError
-from pyroute2.netlink import Marshal
 from pyroute2.netlink import NLM_F_REQUEST
-from pyroute2.netlink.client import Netlink
+from pyroute2.netlink.client import GenericNetlink
 from pyroute2.netlink.generic import nla
 from pyroute2.netlink.generic import genlmsg
-from pyroute2.netlink.generic import ctrlmsg
 
 TASKSTATS_CMD_UNSPEC = 0      # Reserved
 TASKSTATS_CMD_GET = 1         # user->kernel request/get-response
 TASKSTATS_CMD_NEW = 2
 
-NLMSG_MIN_TYPE = 0x10
-
-GENL_NAMSIZ = 16    # length of family name
-GENL_MIN_ID = NLMSG_MIN_TYPE
-GENL_MAX_ID = 1023
-
-GENL_ADMIN_PERM = 0x01
-GENL_CMD_CAP_DO = 0x02
-GENL_CMD_CAP_DUMP = 0x04
-GENL_CMD_CAP_HASPOL = 0x08
-
-#
-# List of reserved static generic netlink identifiers:
-#
-GENL_ID_GENERATE = 0
-GENL_ID_CTRL = NLMSG_MIN_TYPE
-
-#
-# Controller
-#
-
-CTRL_CMD_UNSPEC = 0x0
-CTRL_CMD_NEWFAMILY = 0x1
-CTRL_CMD_DELFAMILY = 0x2
-CTRL_CMD_GETFAMILY = 0x3
-CTRL_CMD_NEWOPS = 0x4
-CTRL_CMD_DELOPS = 0x5
-CTRL_CMD_GETOPS = 0x6
-CTRL_CMD_NEWMCAST_GRP = 0x7
-CTRL_CMD_DELMCAST_GRP = 0x8
-CTRL_CMD_GETMCAST_GRP = 0x9  # unused
-
-
-CTRL_ATTR_UNSPEC = 0x0
-CTRL_ATTR_FAMILY_ID = 0x1
-CTRL_ATTR_FAMILY_NAME = 0x2
-CTRL_ATTR_VERSION = 0x3
-CTRL_ATTR_HDRSIZE = 0x4
-CTRL_ATTR_MAXATTR = 0x5
-CTRL_ATTR_OPS = 0x6
-CTRL_ATTR_MCAST_GROUPS = 0x7
-
-CTRL_ATTR_OP_UNSPEC = 0x0
-CTRL_ATTR_OP_ID = 0x1
-CTRL_ATTR_OP_FLAGS = 0x2
-
-CTRL_ATTR_MCAST_GRP_UNSPEC = 0x0
-CTRL_ATTR_MCAST_GRP_NAME = 0x1
-CTRL_ATTR_MCAST_GRP_ID = 0x2
-
 
 class tcmd(genlmsg):
     nla_map = (('TASKSTATS_CMD_ATTR_UNSPEC', 'none'),
@@ -162,27 +110,10 @@ class taskstatsmsg(genlmsg):
         pass
 
 
-class TaskStats(Netlink):
-
-    marshal = Marshal
+class TaskStats(GenericNetlink):
 
     def __init__(self):
-        Netlink.__init__(self)
-        # FIXME
-        self.marshal.msg_map[GENL_ID_CTRL] = ctrlmsg
-        self.prid = self.get_protocol_id('TASKSTATS')
-        self.marshal.msg_map[self.prid] = taskstatsmsg
-
-    def get_protocol_id(self, prid):
-        msg = ctrlmsg()
-        msg['cmd'] = CTRL_CMD_GETFAMILY
-        msg['version'] = 1
-        msg['attrs'].append(['CTRL_ATTR_FAMILY_NAME', prid])
-        response = self.nlm_request(msg, GENL_ID_CTRL,
-                                    msg_flags=NLM_F_REQUEST)[0]
-        prid = [i[1] for i in response['attrs']
-                if i[0] == 'CTRL_ATTR_FAMILY_ID'][0]
-        return prid
+        GenericNetlink.__init__(self, 'TASKSTATS', taskstatsmsg)
 
     def get_pid_stat(self, pid):
         '''
